@@ -2,11 +2,12 @@ package Test2::Event;
 use strict;
 use warnings;
 
-our $VERSION = '1.302067';
-
+our $VERSION = '1.302075';
 
 use Test2::Util::HashBase qw/trace nested in_subtest subtest_id/;
 use Test2::Util::ExternalMeta qw/meta get_meta set_meta delete_meta/;
+use Test2::Util qw(pkg_to_file);
+use Test2::Util::Trace;
 
 sub causes_fail      { 0 }
 sub increments_count { 0 }
@@ -21,6 +22,47 @@ sub sets_plan { () }
 
 sub summary { ref($_[0]) }
 
+sub related {
+    my $self = shift;
+    my ($event) = @_;
+
+    my $tracea = $self->trace  or return undef;
+    my $traceb = $event->trace or return undef;
+
+    my $siga = $tracea->signature or return undef;
+    my $sigb = $traceb->signature or return undef;
+
+    return 1 if $siga eq $sigb;
+    return 0;
+}
+
+sub from_json {
+    my $class = shift;
+    my %p     = @_;
+
+    my $event_pkg = delete $p{__PACKAGE__};
+    require(pkg_to_file($event_pkg));
+
+    if (exists $p{trace}) {
+        $p{trace} = Test2::Util::Trace->from_json(%{$p{trace}});
+    }
+
+    if (exists $p{subevents}) {
+        my @subevents;
+        for my $subevent (@{delete $p{subevents} || []}) {
+            push @subevents, Test2::Event->from_json(%$subevent);
+        }
+        $p{subevents} = \@subevents;
+    }
+
+    return $event_pkg->new(%p);
+}
+
+sub TO_JSON {
+    my $self = shift;
+    return {%$self, __PACKAGE__ => ref $self};
+}
+
 1;
 
 __END__
@@ -174,6 +216,36 @@ If the event is inside a subtest this should have the subtest ID.
 
 If the event is a final subtest event, this should contain the subtest ID.
 
+=item $bool_or_undef = $e->related($e2)
+
+Check if 2 events are related. In this case related means their traces share a
+signature meaning they were created with the same context (or at the very least
+by contexts which share an id, which is the same thing unless someone is doing
+something very bad).
+
+This can be used to reliably link multiple events created by the same tool. For
+instance a failing test like C<ok(0, "fail"> will generate 2 events, one being
+a L<Test2::Event::Ok>, the other being a L<Test2::Event::Diag>, both of these
+events are related having been created under the same context and by the same
+initial tool (though multiple tools may have been nested under the initial
+one).
+
+This will return C<undef> if the relationship cannot be checked, which happens
+if either event has an incomplete or missing trace. This will return C<0> if
+the traces are complete, but do not match. C<1> will be returned if there is a
+match.
+
+=item $hashref = $e->TO_JSON
+
+This returns a hashref suitable for passing to the C<< Test2::Event->from_json
+>> constructor. It is intended for use with the L<JSON> family of modules,
+which will look for a C<TO_JSON> method when C<convert_blessed> is true.
+
+=item $e = Test2::Event->from_json(%$hashref)
+
+Given the hash of data returned by C<< $e->TO_JSON >>, this method returns a
+new event object of the appropriate subclass.
+
 =back
 
 =head1 THIRD PARTY META-DATA
